------------------------------------------------------------------------------
--                                                                          --
--                            Matreshka Project                             --
--                                                                          --
--                           SQL Database Access                            --
--                                                                          --
--                           Testsuite Component                            --
--                                                                          --
------------------------------------------------------------------------------
--                                                                          --
-- Copyright © 2011-2013, Vadim Godunko <vgodunko@gmail.com>                --
-- All rights reserved.                                                     --
--                                                                          --
-- Redistribution and use in source and binary forms, with or without       --
-- modification, are permitted provided that the following conditions       --
-- are met:                                                                 --
--                                                                          --
--  * Redistributions of source code must retain the above copyright        --
--    notice, this list of conditions and the following disclaimer.         --
--                                                                          --
--  * Redistributions in binary form must reproduce the above copyright     --
--    notice, this list of conditions and the following disclaimer in the   --
--    documentation and/or other materials provided with the distribution.  --
--                                                                          --
--  * Neither the name of the Vadim Godunko, IE nor the names of its        --
--    contributors may be used to endorse or promote products derived from  --
--    this software without specific prior written permission.              --
--                                                                          --
-- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS      --
-- "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT        --
-- LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR    --
-- A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT     --
-- HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,   --
-- SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED --
-- TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   --
-- PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF   --
-- LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING     --
-- NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS       --
-- SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.             --
--                                                                          --
------------------------------------------------------------------------------
--  $Revision: 3758 $ $Date: 2013-02-11 11:09:34 +0400 (Mon, 11 Feb 2013) $
------------------------------------------------------------------------------
--  Check whether parameter value specified in upper case works properly.
------------------------------------------------------------------------------
with League.Holders.Integers;
with League.Strings;
with SQL.Databases;
with SQL.Options;
with SQL.Queries;

with Matreshka.Internals.SQL_Drivers.SQLite3.Factory;
pragma Unreferenced (Matreshka.Internals.SQL_Drivers.SQLite3.Factory);

procedure Test_142 is

   function "+"
    (Item : Wide_Wide_String) return League.Strings.Universal_String
       renames League.Strings.To_Universal_String;

   Driver  : constant League.Strings.Universal_String := +"SQLITE3";
   Options : SQL.Options.SQL_Options;

   DB    : aliased SQL.Databases.SQL_Database
     := SQL.Databases.Create (Driver, Options);
   Query : SQL.Queries.SQL_Query := DB.Query;
   Value : League.Holders.Holder;

begin
   DB.Open;

   --  Create table.

   Query.Prepare (+"CREATE TABLE test142 (value INTEGER)");
   Query.Execute;

   --  Insert value.

   Query.Prepare (+"INSERT INTO test142 VALUES (:VALUE)");
   Query.Bind_Value (+":VALUE", League.Holders.Integers.To_Holder (15));
   Query.Execute;

   --  Retrieve value.

   Query.Prepare (+"SELECT value FROM test142");
   Query.Execute;

   if not Query.Next then
      raise Program_Error;
   end if;

   if League.Holders.Is_Empty (Query.Value (1)) then
      raise Program_Error;
   end if;

   DB.Close;
end Test_142;
